`timescale 1ns / 1ps
/* -----------------------------------------------------------------
	Smart-USB Plus ZBT-SSRAM Controller		TAB = 4;
	Verilog-HDL Version

	Synthesis & Fitter tool = "QuartusII9.0"

	FileName = ZBTSSRAM.v
	
	M.YOSHIDA				16.MAR.2009		REV	1.0
----------------------------------------------------------------- */
module	ZBTSSRAM	(	//	Internal AVALON Port
						clk,
						reset_n,

						address,
						chipselect,
						writedata,
						readdata,
						write,
						read,
						byteenable,
						readdatavalid,

						//	External SSRAM Port
		                 ab,
        		         db,
	                	 csn,
	    	             wn,
    		             bwan,
        		         bwbn,
        		         bwcn,
        		         bwdn,
        		         adv,
        		         cken,
        		         ftn,
        		         gn,
        		         lbon,
        		         zz
					);

	parameter	ADRS_WIDTH	=	21;
	parameter	SSRAM_MODE	=	0;

	input							clk;
	input							reset_n;

	input	[ADRS_WIDTH-1:0]		address;
	input							chipselect;
	input	[31:0]					writedata;
	output	[31:0]					readdata;
	input							write;
	input							read;
	input	[3:0]					byteenable;
	output							readdatavalid;

	output	[ADRS_WIDTH-1:0]		ab;
	inout	[31:0]					db;
	output        				   	wn;
	output     			   		   	csn;
	output				           	bwan;
	output      			     	bwbn;
	output      			     	bwcn;
	output     				      	bwdn;
	output     				      	adv;
	output          			 	cken;
	output          			 	ftn;
	output          			 	gn;
	output          			 	lbon;
	output          				zz;


	reg     [ADRS_WIDTH-1:0]		ab;
	reg              				db_ena;
	reg     [31:0]					db_tmp;
	reg     [31:0]					readdata;
	reg								bwan;
	reg								bwbn;
	reg								bwcn;
	reg								bwdn;
	reg								csn;
	reg								wn;
	reg		[SSRAM_MODE+2:0]		readdatavalid_ff;

	wire							adv;
	wire							cken;
	wire							ftn;
	wire							gn;
	wire							lbon;
	wire							zz;



	always	@( posedge clk or negedge reset_n )	    begin
		if( !reset_n )	begin
			ab	<=	{ ADRS_WIDTH{ 1'b0 }};
		end
		else	begin
			ab	<=	address[ADRS_WIDTH-1:0];
		end
    end


	always	@( posedge clk or negedge reset_n )		begin
		if( !reset_n )	begin
			db_ena	<= 1'b0;
		end
		else	begin
			db_ena	<=	chipselect & write;
		end
	end

	assign	db	=	db_tmp;
	always	@( posedge clk or negedge reset_n )		begin
		if( !reset_n )	begin
			db_tmp	<=	32'hzzzzzzzz;
		end
		else if( db_ena )	begin
			db_tmp	<=	writedata;
		end
		else	begin
			db_tmp	<=	32'hzzzzzzzz;
		end
	end

	always	@( posedge clk or negedge reset_n )		begin
		if( !reset_n )	begin
			readdata	<=	32'b0;
		end
		else	begin
			readdata	<=	db;
		end
	end

	always	@( posedge clk or negedge reset_n )		begin
		if( !reset_n )	begin
          bwan	<=	1'b1;
          bwbn	<=	1'b1;
          bwcn	<=	1'b1;
          bwdn	<=	1'b1;
		end
		else	begin
			bwan	<=	~( chipselect & write & byteenable[0] );
			bwbn	<=	~( chipselect & write & byteenable[1] );
			bwcn	<=	~( chipselect & write & byteenable[2] );
			bwdn	<=	~( chipselect & write & byteenable[3] );
		end
	end

	always	@( posedge clk or negedge reset_n )		begin
		if(	!reset_n )	begin
			csn	<=	1'b1;
		end
		else	begin
			csn	<=	~( chipselect );
		end
	end

	always	@( posedge clk or negedge reset_n ) 	begin
		if( !reset_n )	begin
			wn	<=	1'b1;
		end
		else	begin
			wn	<=	~( chipselect & write );
		end
	end

	generate
		if( SSRAM_MODE == 0 )	begin
			//	FlowThrough Mode
			assign	ftn				=	1'b0;
			assign	readdatavalid	=	readdatavalid_ff[2];
			always	@( posedge clk or negedge reset_n ) 	begin
				if( !reset_n )	begin
					readdatavalid_ff	<=	3'b0;
				end
				else	begin
					readdatavalid_ff[0]		<=	chipselect & read;
					readdatavalid_ff[1]		<=	readdatavalid_ff[0];
					readdatavalid_ff[2]		<=	readdatavalid_ff[1];
				end
			end
		end	
		else	begin
			//	Pipeline Mode
			assign	ftn				=	1'b1;
			assign	readdatavalid	=	readdatavalid_ff[3];
			always	@( posedge clk or negedge reset_n ) 	begin
				if( !reset_n )	begin
					readdatavalid_ff	<=	4'b0;
				end
				else	begin
					readdatavalid_ff[0]		<=	chipselect & read;
					readdatavalid_ff[1]		<=	readdatavalid_ff[0];
					readdatavalid_ff[2]		<=	readdatavalid_ff[1];
					readdatavalid_ff[3]		<=	readdatavalid_ff[2];
				end
			end
		end
	endgenerate



	assign	cken	=	1'b0;
	assign	zz		=	1'b0;
	assign	lbon	=	1'b0;
	assign	adv		=	1'b0;
	assign	gn		=	1'b0;

endmodule
